(* Include file containing code to be linked in with the
   PAVT0 unit to provide ANSI fallback support.           *)
(* February  3, 1993 -- Fixed minor color & cursor handling bugs *)

const
  Saved : Array[1..2] of byte = (1,1);
  Reverse : boolean = False;

var
  CurPrm : byte; { current parameter }

{-- forward references to the ANSI procedures -----------------------}

{$F+}
Procedure ANSI_Waiting(ch:char); forward;
Procedure ANSI_Bracket(ch:char); forward;
Procedure ANSI_InParam(ch:char); forward;
{$F-}
Procedure ANSI_Command(ch:char); forward;

{-- code for ANSI interpreter ---------------------------------------}

Procedure ANSI_Waiting(ch:char);
{ employs a cut down TTY for use in Window 0 as well as the first
  stage of ansi interpretation. The AVT/1 parser will usually handle
  this, but it is here so that it works standalone.                  }
begin
  if ch = #27 then
   AVTInterp := ANSI_Bracket
  else
   AvtTTY(ch);
end; { ANSI_Waiting (aka tty) }

Procedure ANSI_Bracket(ch:char);
begin
  if Ch <> '[' Then
   begin
     AvtInterp := ANSI_Waiting;
     ANSI_Waiting(ch); { low level TTY of char }
     exit;
   end;
  CurPrm := 1;
  AVTInterp := ANSI_InParam;
  FillChar(PrmArr,SizeOf(PrmArr),0);
end; { ANSI_Bracket }

Procedure ANSI_InParam(ch:char);
begin
  case ch of
    '0'..'9' : PrmArr[CurPrm] := PrmArr[CurPrm]*10 + (ord(Ch) - ord('0'));
    ';' : if CurPrm < 10 then inc(CurPrm);
   else { case }
    ANSI_Command(ch);
  end; { case ch }
end; { ANSI_InParam }

Procedure ANSI_Command(ch:char);
var
  i : integer;
  a,b : byte;
  qs : Array[1..2] of String[2];
begin
  if fallback then AVTInterp := AVT0_Waiting { return to AVT/1 con }
   else AVTInterp := ANSI_Waiting;
   case ch of
     { Note: the order of commands is optimized for execution speed }
     'm' :                 {sgr}
       begin
         a := AvtWin.cattr;
         for i := 1 to CurPrm do
         begin
           Case PrmArr[i] Of
             0 : a := 7;
             1 : a := a and $FF or $08;
             5 : a := a or $80;
             30..37 : case PrmArr[i] of
                        30 : a := a and $F8{ or $00};
                        31 : a := a and $F8 or $04;
                        32 : a := a and $F8 or $02;
                        33 : a := a and $F8 or $06;
                        34 : a := a and $F8 or $01;
                        35 : a := a and $F8 or $05;
                        36 : a := a and $F8 or $03;
                        37 : a := a and $F8 or $07;
                      end;
             40..47 : case PrmArr[i] of
                        40 : a := a and $8F{ or $00};
                        41 : a := a and $8F or $40;
                        42 : a := a and $8F or $20;
                        43 : a := a and $8F or $60;
                        44 : a := a and $8F or $10;
                        45 : a := a and $8F or $50;
                        46 : a := a and $8F or $30;
                        47 : a := a and $8F or $70;
                      end;
             22..25 : case PrmArr[i] of
                        22 : a := a and $F7{ or $00};
                        24 : a := a and $F8 or $04;
                        25 : a := a and $7F{ or $00};
                      end;
             2  : a := a and $F7 or $00;
             4  : a := a and $F8 or $01;
             7  : a := (a and $88) or $70; { reverse on }
             27 : a := (a and $88) or $07; { reverse off }
             8  : a := a and $f0 or ((a and $f0) shr 4)
           end; { case }
          end;
          AvtWin.cattr := a; { set new attribute }
        end;
     'C' : begin   {cuf}
             if PrmArr[1] = 0 then PrmArr[1] := 1;
              SetXY(WhereX + PrmArr[1], WhereY);
           end;
     'D' : begin   {cub}
             if PrmArr[1] = 0 then PrmArr[1] := 1;
              SetXY(WhereX - PrmArr[1], WhereY);
           end;
     'A' : begin   {cup}
             if PrmArr[1] = 0 then PrmArr[1] := 1;
              SetXY(WhereX,WhereY - PrmArr[1]);
           end;
     'B' : begin   {cud}
             if PrmArr[1] = 0 then PrmArr[1] := 1;
              SetXY(WhereX,WhereY + PrmArr[1]);
           end;
     'H','f' : begin
             if PrmArr[1] = 0 then PrmArr[1] := 1;
             if PrmArr[2] = 0 then PrmArr[2] := 1;
             SetXY(PrmArr[2],PrmArr[1]);  {cup,hvp}
           end;
     'J' : with AvtWin do       {EID}
            case PrmArr[1] of
              2 : begin
                    Clear(cattr,' ');
                    SetXY(1,1);      { True ANSI won't home the Cursor... }
                  end;
              0 : begin  { ClrEos }
                    if WhereY < depth then { to EOS }
                      FillArea(1,succ(WhereY),Width,Depth,cattr,#0);
                    FillArea(WhereX,WhereY,Width,WhereY,cattr,#0); { to EOL }
                  end;
              1 : begin  { ClrBos }
                    if WhereY > 1 then
                      FillArea(1,1,Width,pred(WhereY),cattr,#0);
                    FillArea(1,WhereY,WhereX,WhereY,cattr,#0); { from BOL }
                  end;
            end; { case }
     'K' : with AvtWin do     {eil}
            case PrmArr[1] Of
              0 : FillArea(WhereX,WhereY,width,WhereY,cattr,' '); { DelEol }
              1 : FillArea(1,WhereY,WhereX,WhereY,cattr,' '); { DelBol }
              2 : FillArea(1,WhereY,width,WhereY,cattr,' '); { ClrLine }
            end; { case PrmArr }
     'L' : with AvtWin do   {il }
            Scroll(2,1,WhereY,width,depth,PrmArr[1]);
     'M' : with AvtWin do   {d_l}
            Scroll(1,1,WhereY,width,depth,PrmArr[1]);
     'P' : begin  {dc }
             if PrmArr[1] = 0 then PrmArr[1] := 1;
             with AvtWin do
              Scroll(3,WhereX,WhereY,width,WhereY,PrmArr[1]);
           end;
     '@' : begin  {ic }
             if PrmArr[1] = 0 then PrmArr[1] := 1;
             with AvtWin do
              Scroll(4,WhereX,WhereY,width,WhereY,PrmArr[1]);
           end;
     'h', 'l' :            {sm/rm}
            { BBS style doesn't support mode changes } ;
     'n' : begin           {dsr}
             Str(WhereX,qs[1]);
             Str(WhereY,qs[2]);
             if (PrmArr[1] = 6) then
              Query_Hook(#27+'['+qs[1]+';'+qs[2]+'R'); { return cp }
           end;
     's' : begin           {scp}
             Saved[1] := WhereX;
             Saved[2] := WhereY;
           end;
     'u' : SetXY(Saved[1],Saved[2]); {rcp}
     'p' : ; { keyboard reassignment is dangerous }
   end { case ch } ;
end; { ANSI_Command }

{-- ANSI Init section ------------------------------------------------}

Procedure ANSI_Reset;
begin
  FillChar(PrmArr,SizeOf(PrmArr),0);
  CurPrm := 1;
  Reverse := False;
end;
